@@ -1,6 +1,7 @@
+# noqa: D205,D400
 """
-Atmospheric indicator calculation instances
-===========================================
+Atmospheric indicators
+======================
 
 While the `indices` module stores the computing functions, this module defines Indicator classes and instances that
 include a number of functionalities, such as input validation, unit conversion, output meta-data handling,
