@@ -1,10 +1,11 @@
 # -*- coding: utf-8 -*-
-import abc
+"""Temperature indicator definitions."""
 
 from xclim import indices
-from xclim.core import checks
-from xclim.core.indicator import Indicator
-from xclim.core.indicator import Indicator2D
+from xclim.core import cfchecks
+from xclim.core.indicator import Daily, Daily2D
+from xclim.core.units import check_units
+from xclim.core.utils import wrapped_partial
 
 __all__ = [
     "tn_days_below",
@@ -16,7 +17,9 @@ __all__ = [
     "heat_wave_index",
     "hot_spell_frequency",
     "hot_spell_max_length",
+    "tg_max",
     "tg_mean",
+    "tg_min",
     "tg10p",
     "tg90p",
     "tn_min",
@@ -30,23 +33,32 @@ __all__ = [
     "tx10p",
     "tx90p",
     "daily_temperature_range",
+    "max_daily_temperature_range",
     "daily_temperature_range_variability",
     "extreme_temperature_range",
     "cold_spell_duration_index",
     "cold_spell_days",
+    "cold_spell_frequency",
     "daily_freezethaw_cycles",
     "cooling_degree_days",
     "heating_degree_days",
     "growing_degree_days",
+    "frost_season_length",
     "freshet_start",
     "frost_days",
     "last_spring_frost",
+    "first_day_below",
+    "first_day_above",
     "ice_days",
     "consecutive_frost_days",
     "maximum_consecutive_frost_free_days",
     "growing_season_length",
     "growing_season_end",
     "tropical_nights",
+    "degree_days_exceedance_date",
+    "warm_spell_duration_index",
+    "maximum_consecutive_warm_days",
+    "fire_season",
 ]
 
 
@@ -54,53 +66,49 @@ __all__ = [
 # E.g. http://vocab.nerc.ac.uk/collection/P07/current/BHMHISG2/
 
 
-class Tas(Indicator):
+class Tas(Daily):
     """Class for univariate indices using mean daily temperature as the input."""
 
-    def cfprobe(self, da):
-        checks.check_valid(da, "cell_methods", "time: mean within days")
-        checks.check_valid(da, "standard_name", "air_temperature")
-
-    @abc.abstractmethod
-    def compute(*args, **kwds):
-        """The function computing the indicator."""
+    @staticmethod
+    def cfcheck(tas):
+        cfchecks.check_valid(tas, "cell_methods", "*time: mean within days*")
+        cfchecks.check_valid(tas, "standard_name", "air_temperature")
 
 
-class Tasmin(Indicator):
+class Tasmin(Daily):
     """Class for univariate indices using min daily temperature as the input."""
 
-    def cfprobe(self, da):
-        checks.check_valid(da, "cell_methods", "time: minimum within days")
-        checks.check_valid(da, "standard_name", "air_temperature")
-
-    @abc.abstractmethod
-    def compute(*args, **kwds):
-        """The function computing the indicator."""
+    @staticmethod
+    def cfcheck(tasmin):
+        cfchecks.check_valid(tasmin, "cell_methods", "*time: minimum within days*")
+        cfchecks.check_valid(tasmin, "standard_name", "air_temperature")
 
 
-class Tasmax(Indicator):
+class Tasmax(Daily):
     """Class for univariate indices using max daily temperature as the input."""
 
-    def cfprobe(self, da):
-        checks.check_valid(da, "cell_methods", "time: maximum within days")
-        checks.check_valid(da, "standard_name", "air_temperature")
-
-    @abc.abstractmethod
-    def compute(*args, **kwds):
-        """The function computing the indicator."""
+    @staticmethod
+    def cfcheck(tasmax):
+        cfchecks.check_valid(tasmax, "cell_methods", "*time: maximum within days*")
+        cfchecks.check_valid(tasmax, "standard_name", "air_temperature")
 
 
-class TasminTasmax(Indicator2D):
-    def cfprobe(self, dan, dax):
-        for da in (dan, dax):
-            checks.check_valid(da, "standard_name", "air_temperature")
-        checks.check_valid(dan, "cell_methods", "time: minimum within days")
-        checks.check_valid(dax, "cell_methods", "time: maximum within days")
-        assert dan.units == dax.units
+class Tasx(Daily):
+    """Class for univariate indices using mean daily temperature as the input."""
 
-    @abc.abstractmethod
-    def compute(*args, **kwds):
-        """The function computing the indicator."""
+    @staticmethod
+    def cfcheck(tas):
+        cfchecks.check_valid(tas, "standard_name", "air_temperature")
+
+
+class TasminTasmax(Daily2D):
+    @staticmethod
+    def cfcheck(tasmin, tasmax):
+        for da in (tasmin, tasmax):
+            cfchecks.check_valid(da, "standard_name", "air_temperature")
+        cfchecks.check_valid(tasmin, "cell_methods", "*time: minimum within days*")
+        cfchecks.check_valid(tasmax, "cell_methods", "*time: maximum within days*")
+        check_units(tasmax, tasmin.attrs["units"])
 
 
 tn_days_below = Tasmin(
@@ -108,7 +116,7 @@ tn_days_below = Tasmin(
     units="days",
     standard_name="number_of_days_with_air_temperature_below_threshold",
     long_name="Number of days with Tmin < {thresh}",
-    description="{freq} number of days where daily minimum temperature is below {thresh}",
+    description="{freq} number of days where daily minimum temperature is below {thresh}.",
     cell_methods="time: minimum within days time: sum over days",
     compute=indices.tn_days_below,
 )
@@ -118,7 +126,7 @@ tx_days_above = Tasmax(
     units="days",
     standard_name="number_of_days_with_air_temperature_above_threshold",
     long_name="Number of days with Tmax > {thresh}",
-    description="{freq} number of days where daily maximum temperature exceeds {thresh}",
+    description="{freq} number of days where daily maximum temperature exceeds {thresh}.",
     cell_methods="time: maximum within days time: sum over days",
     compute=indices.tx_days_above,
 )
@@ -129,7 +137,7 @@ tx_tn_days_above = TasminTasmax(
     standard_name="number_of_days_with_air_temperature_above_threshold",
     long_name="Number of days with Tmax > {thresh_tasmax} and Tmin > {thresh_tasmin}",
     description="{freq} number of days where daily maximum temperature exceeds "
-    "{thresh_tasmax} and minimum temperature exceeds {thresh_tasmin}",
+    "{thresh_tasmax} and minimum temperature exceeds {thresh_tasmin}.",
     cell_methods="",
     compute=indices.tx_tn_days_above,
 )
@@ -194,6 +202,7 @@ heat_wave_index = Tasmax(
     compute=indices.heat_wave_index,
 )
 
+
 hot_spell_frequency = Tasmax(
     identifier="hot_spell_frequency",
     units="",
@@ -230,6 +239,26 @@ tg_mean = Tas(
     compute=indices.tg_mean,
 )
 
+tg_max = Tas(
+    identifier="tg_max",
+    units="K",
+    standard_name="air_temperature",
+    long_name="Maximum daily mean temperature",
+    description="{freq} maximum of daily mean temperature.",
+    cell_methods="time: mean within days time: maximum over days",
+    compute=indices.tg_max,
+)
+
+tg_min = Tas(
+    identifier="tg_min",
+    units="K",
+    standard_name="air_temperature",
+    long_name="Minimum daily mean temperature",
+    description="{freq} minimum of daily mean temperature.",
+    cell_methods="time: mean within days time: minimum over days",
+    compute=indices.tg_min,
+)
+
 tx_mean = Tasmax(
     identifier="tx_mean",
     units="K",
@@ -291,13 +320,25 @@ tn_min = Tasmin(
 )
 
 daily_temperature_range = TasminTasmax(
+    title="Mean of daily temperature range.",
     identifier="dtr",
     units="K",
     standard_name="air_temperature",
     long_name="Mean Diurnal Temperature Range",
-    description="{freq} mean diurnal temperature range",
+    description="{freq} mean diurnal temperature range.",
     cell_methods="time range within days time: mean over days",
-    compute=indices.daily_temperature_range,
+    compute=wrapped_partial(indices.daily_temperature_range, op="mean"),
+)
+
+max_daily_temperature_range = TasminTasmax(
+    title="Maximum of daily temperature range.",
+    identifier="dtrmax",
+    units="K",
+    standard_name="air_temperature",
+    long_name="Maximum Diurnal Temperature Range",
+    description="{freq} maximum diurnal temperature range.",
+    cell_methods="time range within days time: max over days",
+    compute=wrapped_partial(indices.daily_temperature_range, op="max"),
 )
 
 daily_temperature_range_variability = TasminTasmax(
@@ -329,8 +370,7 @@ cold_spell_duration_index = Tasmin(
     var_name="csdi_{window}",
     units="days",
     standard_name="cold_spell_duration_index",
-    long_name="Cold Spell Duration Index, count of days with at "
-    "least {window} consecutive days when Tmin < 10th percentile",
+    long_name="Number of days part of a percentile-defined cold spell",
     description="{freq} number of days with at least {window} consecutive days "
     "where the daily minimum temperature is below the 10th "
     "percentile. The 10th percentile should be computed for "
@@ -343,7 +383,7 @@ cold_spell_days = Tas(
     identifier="cold_spell_days",
     units="days",
     standard_name="cold_spell_days",
-    long_name="cold spell index",
+    long_name="Number of days part of a cold spell",
     description="{freq} number of days that are part of a cold spell, defined as {window} "
     "or more consecutive days with mean daily "
     "temperature below  {thresh}.",
@@ -351,6 +391,19 @@ cold_spell_days = Tas(
     compute=indices.cold_spell_days,
 )
 
+cold_spell_frequency = Tas(
+    identifier="cold_spell_frequency",
+    units="",
+    standard_name="cold_spell_frequency",
+    long_name="Number of cold spell events",
+    description="{freq} number cold spell events, defined as {window} "
+    "or more consecutive days with mean daily "
+    "temperature below  {thresh}.",
+    cell_methods="",
+    compute=indices.cold_spell_frequency,
+)
+
+
 daily_freezethaw_cycles = TasminTasmax(
     identifier="dlyfrzthw",
     units="days",
@@ -412,6 +465,19 @@ frost_days = Tasmin(
     compute=indices.frost_days,
 )
 
+frost_season_length = Tasmin(
+    identifier="frost_season_length",
+    units="days",
+    standard_name="days_with_air_temperature_below_threshold",
+    long_name="Length of the frost season",
+    description="{freq} number of days between the first occurrence of at least "
+    "{window} consecutive days with minimum daily temperature below freezing and "
+    "the first occurrence of at least {window} consecutive days with "
+    "minimuim daily temperature above freezing after {mid_date}.",
+    cell_methods="time: minimum within days time: sum over days",
+    compute=wrapped_partial(indices.frost_season_length, thresh="0 degC"),
+)
+
 last_spring_frost = Tasmin(
     identifier="last_spring_frost",
     units="",
@@ -422,6 +488,25 @@ last_spring_frost = Tasmin(
     compute=indices.last_spring_frost,
 )
 
+first_day_below = Tasmin(
+    identifier="first_day_below",
+    units="",
+    standard_name="day_of_year",
+    long_name="First day of year with temperature below {thresh}",
+    description="First day of year with temperature below {thresh} for at least {window} days.",
+    compute=indices.first_day_below,
+)
+
+first_day_above = Tasmin(
+    identifier="first_day_above",
+    units="",
+    standard_name="day_of_year",
+    long_name="First day of year with temperature above {thresh}",
+    description="First day of year with temperature above {thresh} for at least {window} days.",
+    compute=indices.first_day_above,
+)
+
+
 ice_days = Tasmax(
     identifier="ice_days",
     standard_name="days_with_air_temperature_below_threshold",
@@ -436,11 +521,11 @@ consecutive_frost_days = Tasmin(
     identifier="consecutive_frost_days",
     units="days",
     standard_name="spell_length_of_days_with_air_temperature_below_threshold",
-    long_name="Maximum number of consecutive days with Tmin < 0C",
+    long_name="Maximum number of consecutive days with Tmin < {thresh}",
     description="{freq} maximum number of consecutive days with "
-    "minimum daily temperature below 0℃",
+    "minimum daily temperature below {thresh}",
     cell_methods="time: min within days time: maximum over days",
-    compute=indices.consecutive_frost_days,
+    compute=indices.maximum_consecutive_frost_days,
 )
 
 maximum_consecutive_frost_free_days = Tasmin(
@@ -460,7 +545,7 @@ growing_season_length = Tas(
     standard_name="growing_season_length",
     long_name="ETCCDI Growing Season Length (Tmean > {thresh})",
     description="{freq} number of days between the first occurrence of at least "
-    "six consecutive days with mean daily temperature over {thresh} and "
+    "{window} consecutive days with mean daily temperature over {thresh} and "
     "the first occurrence of at least {window} consecutive days with "
     "mean daily temperature below {thresh} after {mid_date}.",
     cell_methods="",
@@ -496,7 +581,7 @@ tg90p = Tas(
     standard_name="days_with_air_temperature_above_threshold",
     long_name="Number of days when Tmean > 90th percentile",
     description="{freq} number of days with mean daily temperature above the 90th percentile."
-    "The 90th percentile is to be computed for a 5 day window centered on each calendar day "
+    "The 90th percentile is to be computed for a 5 day moving window centered on each calendar day "
     "for a reference period.",
     cell_methods="time: mean within days time: sum over days",
     compute=indices.tg90p,
@@ -508,7 +593,7 @@ tg10p = Tas(
     standard_name="days_with_air_temperature_below_threshold",
     long_name="Number of days when Tmean < 10th percentile",
     description="{freq} number of days with mean daily temperature below the 10th percentile."
-    "The 10th percentile is to be computed for a 5 day window centered on each calendar day "
+    "The 10th percentile is to be computed for a 5 day moving window centered on each calendar day "
     "for a reference period.",
     cell_methods="time: mean within days time: sum over days",
     compute=indices.tg10p,
@@ -520,7 +605,7 @@ tx90p = Tasmax(
     standard_name="days_with_air_temperature_above_threshold",
     long_name="Number of days when Tmax > 90th percentile",
     description="{freq} number of days with maximum daily temperature above the 90th percentile."
-    "The 90th percentile is to be computed for a 5 day window centered on each calendar day "
+    "The 90th percentile is to be computed for a 5 day moving window centered on each calendar day "
     "for a reference period.",
     cell_methods="time: maximum within days time: sum over days",
     compute=indices.tx90p,
@@ -532,7 +617,7 @@ tx10p = Tasmax(
     standard_name="days_with_air_temperature_below_threshold",
     long_name="Number of days when Tmax < 10th percentile",
     description="{freq} number of days with maximum daily temperature below the 10th percentile."
-    "The 10th percentile is to be computed for a 5 day window centered on each calendar day "
+    "The 10th percentile is to be computed for a 5 day moving window centered on each calendar day "
     "for a reference period.",
     cell_methods="time: maximum within days time: sum over days",
     compute=indices.tx10p,
@@ -544,7 +629,7 @@ tn90p = Tasmin(
     standard_name="days_with_air_temperature_above_threshold",
     long_name="Number of days when Tmin > 90th percentile",
     description="{freq} number of days with minimum daily temperature above the 90th percentile."
-    "The 90th percentile is to be computed for a 5 day window centered on each calendar day "
+    "The 90th percentile is to be computed for a 5 day moving window centered on each calendar day "
     "for a reference period.",
     cell_methods="time: minimum within days time: sum over days",
     compute=indices.tn90p,
@@ -556,8 +641,49 @@ tn10p = Tasmin(
     standard_name="days_with_air_temperature_below_threshold",
     long_name="Number of days when Tmin < 10th percentile",
     description="{freq} number of days with minimum daily temperature below the 10th percentile."
-    "The 10th percentile is to be computed for a 5 day window centered on each calendar day "
+    "The 10th percentile is to be computed for a 5 day moving window centered on each calendar day "
     "for a reference period.",
     cell_methods="time: minimum within days time: sum over days",
     compute=indices.tn10p,
 )
+
+
+degree_days_exceedance_date = Tas(
+    identifier="degree_days_exceedance_date",
+    units="",
+    standard_name="day_of_year",
+    long_name="Day of year when cumulative degree days exceed {sum_thresh}.",
+    description="Day of year when the integral of degree days (tmean {op} {thresh})"
+    " exceeds {sum_thresh}, the cumulative sum starts on {after_date}.",
+    cell_methods="",
+    compute=indices.degree_days_exceedance_date,
+)
+
+
+warm_spell_duration_index = Tasmax(
+    identifier="warm_spell_duration_index",
+    description="{freq} total number of days within spells of at least {window} days"
+    " with tmax above the 90th daily percentile.",
+    units="days",
+    standard_name="number_of_days_with_air_temperature_above_threshold",
+    cell_methods="time: sum over days",
+    compute=indices.warm_spell_duration_index,
+)
+
+
+maximum_consecutive_warm_days = Tasmax(
+    identifier="maximum_consecutive_warm_days",
+    description="{freq} longest spell of consecutive days with Tmax above {thresh}.",
+    units="days",
+    standard_name="spell_length_of_days_with_air_temperature_above_threshold",
+    cell_methods="time: maximum over days",
+    compute=indices.maximum_consecutive_tx_days,
+)
+
+
+fire_season = Tasx(
+    identifier="fire_season",
+    description="Fire season mask, computed with method {method}.",
+    units="",
+    compute=indices.fire_season,
+)
