@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
+# noqa: D205,D400
 """
 ===============
 Indices library
 ===============
 
 This module contains climate indices functions operating on `xarray.DataArray`. Most of these
-functions operate on daily time series, but some might accept other sampling frequencies as well. All
+functions operate on daily time series, but might accept other sampling frequencies as well. All
 functions perform units checks to make sure that inputs have the expected dimensions (for example
-have units of temperature, whether it is celsius, kelvin or fahrenheit), and the the `units`
+have units of temperature, whether it is celsius, kelvin or fahrenheit), and set the `units`
 attribute of the output DataArray.
 
 The `calendar`, `fwi`, `generic`, `run_length` and `utils` submodule provide helpers to simplify
@@ -21,10 +22,13 @@ the implementation of the indices.
     :mod:`xclim.indicators.land` and :mod:`xclim.indicators.seaIce` modules, documented in :ref:`Climate Indicators`.
 
 """
+from ._anuclim import *
 from ._conversion import *
+from ._hydrology import *
 from ._multivariate import *
 from ._simple import *
 from ._threshold import *
+from .fwi import drought_code, fire_season, fire_weather_indexes
 
 """
 Notes for docstring
@@ -43,22 +47,36 @@ about the data or methods used to produce it).
 The third and fourth sections are the **Parameters** and **Returns** sections describing the input and output values
 respectively.
 
+The following example shows the structure of an indice definition:
+
 .. code-block:: python
 
-   Parameters
-   ----------
-   <standard_name> : xarray.DataArray
-     <Long_name> of variable [acceptable units].
-   threshold : Union[str, float, int]
-     Description of the threshold / units.
-     e.g. The 10th percentile of historical temperature [K].
-   freq : Optional[str]
-     Resampling frequency.
+   @declare_units(var1="[units dimension]", thresh="[units dimension]")
+   def indice_name(var1: xr.DataArray, thresh: str = "0 degC", freq: str = "YS"):
+       \"\"\"
+       The first line is the title
 
-   Returns
-   -------
-   xarray.DataArray
-     Output's <long_name> [units]
+       The second paragraph is the abstract.
+
+       Parameters
+       ----------
+       var1 : xarray.DataArray
+         Description of variable (no need to specify units, the signature and decorator carry this information).
+         <var1> is a short name like "tas", "pr" or "sfcWind".
+       thresh : str
+         Description of the threshold (no need to specify units or the default again).
+         Parameters required to run the computation must always have a working default value.
+       freq : str
+         Resampling frequency. (the signature carries the default value information)
+
+       Returns
+       -------
+       <var_name> : xarray.DataArray, [output units dimension]
+         Output's <long_name>
+         Specifying <var_name> is optional.
+       \"\"\"
+       <body of the function>
+       # Don't forget to explicitly handle the units!
 
 The next sections would be **Notes** and **References**:
 
