@@ -1,13 +1,9 @@
+# noqa: D100
 import xarray
 
-from xclim.core.units import convert_units_to
-from xclim.core.units import declare_units
-from xclim.core.units import pint_multiply
-from xclim.core.units import units
-from xclim.indices import run_length as rl
-
-xarray.set_options(enable_cftimeindex=True)  # Set xarray to use cftimeindex
+from xclim.core.units import convert_units_to, declare_units, rate2amount, to_agg_units
 
+from .generic import threshold_count
 
 # Frequencies : YS: year start, QS-DEC: seasons starting in december, MS: month start
 # See http://pandas.pydata.org/pandas-docs/stable/timeseries.html#offset-aliases
@@ -26,16 +22,15 @@ __all__ = [
     "tx_max",
     "tx_mean",
     "tx_min",
-    "base_flow_index",
-    "consecutive_frost_days",
     "frost_days",
     "ice_days",
     "max_1day_precipitation_amount",
     "max_n_day_precipitation_amount",
+    "max_pr_intensity",
 ]
 
 
-@declare_units("[temperature]", tas="[temperature]")
+@declare_units(tas="[temperature]")
 def tg_max(tas: xarray.DataArray, freq: str = "YS"):
     r"""Highest mean temperature.
 
@@ -44,13 +39,13 @@ def tg_max(tas: xarray.DataArray, freq: str = "YS"):
     Parameters
     ----------
     tas : xarray.DataArray
-      Mean daily temperature [℃] or [K]
+      Mean daily temperature.
     freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
+      Resampling frequency.
 
     Returns
     -------
-    xarray.DataArray
+    xarray.DataArray, [same units as tas]
       Maximum of daily minimum temperature.
 
     Notes
@@ -62,11 +57,10 @@ def tg_max(tas: xarray.DataArray, freq: str = "YS"):
 
         TNx_j = max(TN_{ij})
     """
-
     return tas.resample(time=freq).max(dim="time", keep_attrs=True)
 
 
-@declare_units("[temperature]", tas="[temperature]")
+@declare_units(tas="[temperature]")
 def tg_mean(tas: xarray.DataArray, freq: str = "YS"):
     r"""Mean of daily average temperature.
 
@@ -75,13 +69,13 @@ def tg_mean(tas: xarray.DataArray, freq: str = "YS"):
     Parameters
     ----------
     tas : xarray.DataArray
-      Mean daily temperature [℃] or [K]
+      Mean daily temperature.
     freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
+      Resampling frequency.
 
     Returns
     -------
-    xarray.DataArray
+    xarray.DataArray, [same units as tas]
       The mean daily temperature at the given time frequency
 
     Notes
@@ -93,38 +87,35 @@ def tg_mean(tas: xarray.DataArray, freq: str = "YS"):
 
        TG_p = \frac{\sum_{i=a}^{b} TN_i}{b - a + 1}
 
-
     Examples
     --------
-
     The following would compute for each grid cell of file `tas.day.nc` the mean temperature
     at the seasonal frequency, ie DJF, MAM, JJA, SON, DJF, etc.:
 
-    >>> import xarray as xr
-    >>> t = xr.open_dataset('tas.day.nc')
-    >>> tg = tm_mean(t, freq="QS-DEC")
+    >>> from xclim.indices import tg_mean
+    >>> t = xr.open_dataset(path_to_tas_file).tas
+    >>> tg = tg_mean(t, freq="QS-DEC")
     """
-
-    arr = tas.resample(time=freq) if freq else tas
+    arr = tas.resample(time=freq)
     return arr.mean(dim="time", keep_attrs=True)
 
 
-@declare_units("[temperature]", tas="[temperature]")
+@declare_units(tas="[temperature]")
 def tg_min(tas: xarray.DataArray, freq: str = "YS"):
-    r"""Lowest mean temperature
+    r"""Lowest mean temperature.
 
     Minimum of daily mean temperature.
 
     Parameters
     ----------
     tas : xarray.DataArray
-      Mean daily temperature [℃] or [K]
+      Mean daily temperature.
     freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
+      Resampling frequency.
 
     Returns
     -------
-    xarray.DataArray
+    xarray.DataArray, [same units as tas]
       Minimum of daily minimum temperature.
 
     Notes
@@ -136,11 +127,10 @@ def tg_min(tas: xarray.DataArray, freq: str = "YS"):
 
         TGn_j = min(TG_{ij})
     """
-
     return tas.resample(time=freq).min(dim="time", keep_attrs=True)
 
 
-@declare_units("[temperature]", tasmin="[temperature]")
+@declare_units(tasmin="[temperature]")
 def tn_max(tasmin: xarray.DataArray, freq: str = "YS"):
     r"""Highest minimum temperature.
 
@@ -149,13 +139,13 @@ def tn_max(tasmin: xarray.DataArray, freq: str = "YS"):
     Parameters
     ----------
     tasmin : xarray.DataArray
-      Minimum daily temperature [℃] or [K]
+      Minimum daily temperature.
     freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
+      Resampling frequency.
 
     Returns
     -------
-    xarray.DataArray
+    xarray.DataArray, [same units as tasmin]
       Maximum of daily minimum temperature.
 
     Notes
@@ -167,11 +157,10 @@ def tn_max(tasmin: xarray.DataArray, freq: str = "YS"):
 
         TNx_j = max(TN_{ij})
     """
-
     return tasmin.resample(time=freq).max(dim="time", keep_attrs=True)
 
 
-@declare_units("[temperature]", tasmin="[temperature]")
+@declare_units(tasmin="[temperature]")
 def tn_mean(tasmin: xarray.DataArray, freq: str = "YS"):
     r"""Mean minimum temperature.
 
@@ -180,13 +169,13 @@ def tn_mean(tasmin: xarray.DataArray, freq: str = "YS"):
     Parameters
     ----------
     tasmin : xarray.DataArray
-      Minimum daily temperature [℃] or [K]
+      Minimum daily temperature.
     freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
+      Resampling frequency.
 
     Returns
     -------
-    xarray.DataArray
+    xarray.DataArray, [same units as tasmin]
       Mean of daily minimum temperature.
 
     Notes
@@ -198,27 +187,26 @@ def tn_mean(tasmin: xarray.DataArray, freq: str = "YS"):
 
         TN_{ij} = \frac{ \sum_{i=1}^{I} TN_{ij} }{I}
     """
-
-    arr = tasmin.resample(time=freq) if freq else tasmin
+    arr = tasmin.resample(time=freq)
     return arr.mean(dim="time", keep_attrs=True)
 
 
-@declare_units("[temperature]", tasmin="[temperature]")
+@declare_units(tasmin="[temperature]")
 def tn_min(tasmin: xarray.DataArray, freq: str = "YS"):
-    r"""Lowest minimum temperature
+    r"""Lowest minimum temperature.
 
     Minimum of daily minimum temperature.
 
     Parameters
     ----------
     tasmin : xarray.DataArray
-      Minimum daily temperature [℃] or [K]
+      Minimum daily temperature.
     freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
+      Resampling frequency.
 
     Returns
     -------
-    xarray.DataArray
+    xarray.DataArray, [same units as tasmin]
       Minimum of daily minimum temperature.
 
     Notes
@@ -230,26 +218,25 @@ def tn_min(tasmin: xarray.DataArray, freq: str = "YS"):
 
         TNn_j = min(TN_{ij})
     """
-
     return tasmin.resample(time=freq).min(dim="time", keep_attrs=True)
 
 
-@declare_units("[temperature]", tasmax="[temperature]")
+@declare_units(tasmax="[temperature]")
 def tx_max(tasmax: xarray.DataArray, freq: str = "YS"):
-    r"""Highest max temperature
+    r"""Highest max temperature.
 
     The maximum value of daily maximum temperature.
 
     Parameters
     ----------
     tasmax : xarray.DataArray
-      Maximum daily temperature [℃] or [K]
+      Maximum daily temperature.
     freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
+      Resampling frequency.
 
     Returns
     -------
-    xarray.DataArray
+    xarray.DataArray, [same units as tasmax]
       Maximum value of daily maximum temperature.
 
     Notes
@@ -261,26 +248,25 @@ def tx_max(tasmax: xarray.DataArray, freq: str = "YS"):
 
         TXx_j = max(TX_{ij})
     """
-
     return tasmax.resample(time=freq).max(dim="time", keep_attrs=True)
 
 
-@declare_units("[temperature]", tasmax="[temperature]")
+@declare_units(tasmax="[temperature]")
 def tx_mean(tasmax: xarray.DataArray, freq: str = "YS"):
-    r"""Mean max temperature
+    r"""Mean max temperature.
 
     The mean of daily maximum temperature.
 
     Parameters
     ----------
     tasmax : xarray.DataArray
-      Maximum daily temperature [℃] or [K]
+      Maximum daily temperature.
     freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
+      Resampling frequency.
 
     Returns
     -------
-    xarray.DataArray
+    xarray.DataArray, [same units as tasmax]
       Mean of daily maximum temperature.
 
     Notes
@@ -292,27 +278,26 @@ def tx_mean(tasmax: xarray.DataArray, freq: str = "YS"):
 
         TX_{ij} = \frac{ \sum_{i=1}^{I} TX_{ij} }{I}
     """
-
-    arr = tasmax.resample(time=freq) if freq else tasmax
+    arr = tasmax.resample(time=freq)
     return arr.mean(dim="time", keep_attrs=True)
 
 
-@declare_units("[temperature]", tasmax="[temperature]")
+@declare_units(tasmax="[temperature]")
 def tx_min(tasmax: xarray.DataArray, freq: str = "YS"):
-    r"""Lowest max temperature
+    r"""Lowest max temperature.
 
     The minimum of daily maximum temperature.
 
     Parameters
     ----------
     tasmax : xarray.DataArray
-      Maximum daily temperature [℃] or [K]
+      Maximum daily temperature.
     freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
+      Resampling frequency.
 
     Returns
     -------
-    xarray.DataArray
+    xarray.DataArray, [same units as tasmax]
       Minimum of daily maximum temperature.
 
     Notes
@@ -324,115 +309,27 @@ def tx_min(tasmax: xarray.DataArray, freq: str = "YS"):
 
         TXn_j = min(TX_{ij})
     """
-
     return tasmax.resample(time=freq).min(dim="time", keep_attrs=True)
 
 
-@declare_units("", q="[discharge]")
-def base_flow_index(q: xarray.DataArray, freq: str = "YS"):
-    r"""Base flow index
-
-    Return the base flow index, defined as the minimum 7-day average flow divided by the mean flow.
-
-    Parameters
-    ----------
-    q : xarray.DataArray
-      Rate of river discharge [m³/s]
-    freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
-
-    Returns
-    -------
-    xarray.DataArray
-      Base flow index.
-
-    Notes
-    -----
-    Let :math:`\mathbf{q}=q_0, q_1, \ldots, q_n` be the sequence of daily discharge and :math:`\overline{\mathbf{q}}`
-    the mean flow over the period. The base flow index is given by:
-
-    .. math::
-
-       \frac{\min(\mathrm{CMA}_7(\mathbf{q}))}{\overline{\mathbf{q}}}
-
-
-    where :math:`\mathrm{CMA}_7` is the seven days moving average of the daily flow:
-
-    .. math::
-
-       \mathrm{CMA}_7(q_i) = \frac{\sum_{j=i-3}^{i+3} q_j}{7}
-
-    """
-
-    m7 = (
-        q.rolling(time=7, center=True)
-        .mean(allow_lazy=True, skipna=False)
-        .resample(time=freq)
-    )
-    mq = q.resample(time=freq)
-
-    m7m = m7.min(dim="time")
-    return m7m / mq.mean(dim="time")
-
-
-@declare_units("days", tasmin="[temperature]")
-def consecutive_frost_days(tasmin: xarray.DataArray, freq: str = "AS-JUL"):
-    r"""Maximum number of consecutive frost days (Tmin < 0℃).
-
-    Resample the daily minimum temperature series by computing the maximum number
-    of days below the freezing point over each period.
-
-    Parameters
-    ----------
-    tasmin : xarray.DataArray
-      Minimum daily temperature values [℃] or [K]
-    freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
-
-    Returns
-    -------
-    xarray.DataArray
-      The maximum number of consecutive days below the freezing point.
-
-    Notes
-    -----
-    Let :math:`\mathbf{x}=x_0, x_1, \ldots, x_n` be a daily minimum temperature series and
-    :math:`\mathbf{s}` be the sorted vector of indices :math:`i` where :math:`[p_i < 0^\circ C] \neq [p_{i+1} <
-    0^\circ C]`, that is, the days when the temperature crosses the freezing point.
-    Then the maximum number of consecutive frost days is given by
-
-    .. math::
-
-       \max(\mathbf{d}) \quad \mathrm{where} \quad d_j = (s_j - s_{j-1}) [x_{s_j} > 0^\circ C]
-
-    where :math:`[P]` is 1 if :math:`P` is true, and 0 if false. Note that this formula does not handle sequences at
-    the start and end of the series, but the numerical algorithm does.
-    """
-    tu = units.parse_units(tasmin.attrs["units"].replace("-", "**-"))
-    fu = "degC"
-    frz = 0
-    if fu != tu:
-        frz = units.convert(frz, fu, tu)
-    group = (tasmin < frz).resample(time=freq)
-    return group.map(rl.longest_run, dim="time")
-
-
-@declare_units("days", tasmin="[temperature]")
-def frost_days(tasmin: xarray.DataArray, freq: str = "YS"):
-    r"""Frost days index
+@declare_units(tasmin="[temperature]", thresh="[temperature]")
+def frost_days(tasmin: xarray.DataArray, thresh: str = "0 degC", freq: str = "YS"):
+    r"""Frost days index.
 
     Number of days where daily minimum temperatures are below 0℃.
 
     Parameters
     ----------
     tasmin : xarray.DataArray
-      Minimum daily temperature [℃] or [K]
+      Minimum daily temperature.
+    thresh : str
+      Freezing temperature.
     freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
+      Resampling frequency.
 
     Returns
     -------
-    xarray.DataArray
+    xarray.DataArray, [time]
       Frost days index.
 
     Notes
@@ -444,31 +341,31 @@ def frost_days(tasmin: xarray.DataArray, freq: str = "YS"):
 
         TN_{ij} < 0℃
     """
-    tu = units.parse_units(tasmin.attrs["units"].replace("-", "**-"))
-    fu = "degC"
-    frz = 0
-    if fu != tu:
-        frz = units.convert(frz, fu, tu)
-    f = (tasmin < frz) * 1
-    return f.resample(time=freq).sum(dim="time")
+    frz = convert_units_to(thresh, tasmin)
+    out = threshold_count(tasmin, "<", frz, freq)
+    return to_agg_units(out, tasmin, "count")
 
 
-@declare_units("days", tasmax="[temperature]")
-def ice_days(tasmax: xarray.DataArray, freq: str = "YS"):
-    r"""Number of ice/freezing days
+@declare_units(tasmax="[temperature]", thresh="[temperature]")
+def ice_days(
+    tasmax: xarray.DataArray, thresh: str = "0 degC", freq: str = "YS"
+):  # noqa: D401
+    r"""Number of ice/freezing days.
 
     Number of days where daily maximum temperatures are below 0℃.
 
     Parameters
     ----------
     tasmax : xarray.DataArray
-      Maximum daily temperature [℃] or [K]
+      Maximum daily temperature.
+    thresh : str
+      Freezing temperature.
     freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
+      Resampling frequency.
 
     Returns
     -------
-    xarray.DataArray
+    xarray.DataArray, [time]
       Number of ice/freezing days.
 
     Notes
@@ -480,16 +377,12 @@ def ice_days(tasmax: xarray.DataArray, freq: str = "YS"):
 
         TX_{ij} < 0℃
     """
-    tu = units.parse_units(tasmax.attrs["units"].replace("-", "**-"))
-    fu = "degC"
-    frz = 0
-    if fu != tu:
-        frz = units.convert(frz, fu, tu)
-    f = (tasmax < frz) * 1
-    return f.resample(time=freq).sum(dim="time")
+    frz = convert_units_to(thresh, tasmax)
+    out = threshold_count(tasmax, "<", frz, freq)
+    return to_agg_units(out, tasmax, "count")
 
 
-@declare_units("mm/day", pr="[precipitation]")
+@declare_units(pr="[precipitation]")
 def max_1day_precipitation_amount(pr: xarray.DataArray, freq: str = "YS"):
     r"""Highest 1-day precipitation amount for a period (frequency).
 
@@ -498,14 +391,14 @@ def max_1day_precipitation_amount(pr: xarray.DataArray, freq: str = "YS"):
     Parameters
     ----------
     pr : xarray.DataArray
-      Daily precipitation values [Kg m-2 s-1] or [mm]
+      Daily precipitation values.
     freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
+      Resampling frequency.
 
     Returns
     -------
-    xarray.DataArray
-      The highest 1-day precipitation value at the given time frequency.
+    xarray.DataArray, [same units as pr]
+      The highest 1-period precipitation flux value at the given time frequency.
 
     Notes
     -----
@@ -517,20 +410,20 @@ def max_1day_precipitation_amount(pr: xarray.DataArray, freq: str = "YS"):
 
     Examples
     --------
-    The following would compute for each grid cell of file `pr.day.nc` the highest 1-day total
-    at an annual frequency:
+    >>> from xclim.indices import max_1day_precipitation_amount
 
-    >>> import xarray as xr
-    >>> pr = xr.open_dataset('pr.day.nc').pr
+    # The following would compute for each grid cell the highest 1-day total
+    # at an annual frequency:
+    >>> pr = xr.open_dataset(path_to_pr_file).pr
     >>> rx1day = max_1day_precipitation_amount(pr, freq="YS")
     """
-
-    out = pr.resample(time=freq).max(dim="time", keep_attrs=True)
-    return convert_units_to(out, "mm/day", "hydro")
+    return pr.resample(time=freq).max(dim="time", keep_attrs=True)
 
 
-@declare_units("mm", pr="[precipitation]")
-def max_n_day_precipitation_amount(pr, window: int = 1, freq: str = "YS"):
+@declare_units(pr="[precipitation]")
+def max_n_day_precipitation_amount(
+    pr: xarray.DataArray, window: int = 1, freq: str = "YS"
+):
     r"""Highest precipitation amount cumulated over a n-day moving window.
 
     Calculate the n-day rolling sum of the original daily total precipitation series
@@ -539,32 +432,67 @@ def max_n_day_precipitation_amount(pr, window: int = 1, freq: str = "YS"):
     Parameters
     ----------
     pr : xarray.DataArray
-      Daily precipitation values [Kg m-2 s-1] or [mm]
+      Daily precipitation values.
     window : int
       Window size in days.
     freq : str
-      Resampling frequency; Defaults to "YS" (yearly).
+      Resampling frequency.
 
     Returns
     -------
-    xarray.DataArray
-      The highest cumulated n-day precipitation value at the given time frequency.
+    xarray.DataArray, [length]
+      The highest cumulated n-period precipitation value at the given time frequency.
 
     Examples
     --------
-    The following would compute for each grid cell of file `pr.day.nc` the highest 5-day total precipitation
-    at an annual frequency:
+    >>> from xclim.indices import max_n_day_precipitation_amount
 
-    >>> import xarray as xr
-    >>> pr = xr.open_dataset('pr.day.nc').pr
-    >>> window = 5
-    >>> output = max_n_day_precipitation_amount(pr, window=window, freq="YS")
+    # The following would compute for each grid cell the highest 5-day total precipitation
+    #at an annual frequency:
+    >>> pr = xr.open_dataset(path_to_pr_file).pr
+    >>> out = max_n_day_precipitation_amount(pr, window=5, freq="YS")
     """
-
-    # rolling sum of the values
-    arr = pr.rolling(time=window).sum(allow_lazy=True, skipna=False)
+    # Rolling sum of the values
+    pram = rate2amount(pr)
+    arr = pram.rolling(time=window).sum(skipna=False)
     out = arr.resample(time=freq).max(dim="time", keep_attrs=True)
 
+    out.attrs["units"] = pram.units
+    return out
+
+
+@declare_units(pr="[precipitation]")
+def max_pr_intensity(pr: xarray.DataArray, window: int = 1, freq: str = "YS"):
+    r"""Highest precipitation intensity over a n-hour moving window.
+
+    Calculate the n-hour rolling average of the original hourly total precipitation series
+    and determine the maximum value over each period.
+
+    Parameters
+    ----------
+    pr : xarray.DataArray
+      Hourly precipitation values.
+    window : int
+      Window size in hours.
+    freq : str
+      Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray, [same units as pr]
+      The highest cumulated n-hour precipitation intensity at the given time frequency.
+
+    Examples
+    --------
+    >>> from xclim.indices import max_pr_intensity
+
+    # The following would compute the maximum 6-hour precipitation intensity.
+    # at an annual frequency:
+    # TODO
+    """
+    # Rolling sum of the values
+    arr = pr.rolling(time=window).mean(skipna=False)
+    out = arr.resample(time=freq).max(dim="time")
+
     out.attrs["units"] = pr.units
-    # Adjust values and units to make sure they are daily
-    return pint_multiply(out, 1 * units.day, "mm")
+    return out
